<!doctype html><html lang="en"><head><meta charset="utf-8"/><meta name="viewport" content="width=device-width,initial-scale=1"/><title>ASReview LAB - AI-assisted systematic reviews</title><link rel="apple-touch-icon" sizes="180x180" href="/apple-touch-icon.png"/><link rel="icon" type="image/png" sizes="32x32" href="/favicon-32x32.png"/><link rel="icon" type="image/png" sizes="16x16" href="/favicon-16x16.png"/><meta property="og:title" content="ASReview LAB"/><meta property="og:description" content="Smart software for systematic reviews"/><meta property="og:image" content="/social-preview.png"/><meta property="og:url" content=""/><meta name="theme-color" content="#7a6958" media="(prefers-color-scheme: light)"/><meta name="theme-color" content="#a69c8b" media="(prefers-color-scheme: dark)"/><link rel="manifest" href="/app.webmanifest"/><script type="text/javascript">window.api_url = "{{ api_url }}";
      window.asreviewVersion = "{{ asreview_version }}";
      window.authentication = "{{ authentication }}" !== "false";
      window.loginInfo = "{{ login_info }}";
      window.allowAccountCreation = {{ allow_account_creation }};
      window.emailVerification = {{ email_verification }};
      window.oAuthData = {{ oauth | safe }};
      window.postLogoutUrl = "{{ post_logout_url }}";
      window.termsOfAgreement = "%REACT_APP_TERMS_OF_AGREEMENT%" === "true";
      window.termsText = "%REACT_APP_TERMS_TEXT%";</script><script defer="defer" src="/static/js/main.06c9b30e.js"></script><link href="/static/css/main.4bf1134b.css" rel="stylesheet"></head><body><noscript>You need to enable JavaScript to run this app.</noscript><div id="root"></div></body></html>