<p>Hello {{ name }},</p>

<p>
  Use the code below to reset your password. Please note that the code will
  expire in 20 minutes.
</p>

<h2 style="margin: 0; text-align: center; font-size: 24px; color: #333">
  {{ token }}
</h2>

{{ footer_html|safe }}
